#!/usr/bin/env python

import time

def dot (d):
	if not d:
		return ''	# ''.split('.') -> [''] one element list -> \\000
	if d[-1] == '.':
		return d
	return d + '.'		# 'domain.com.'.split('.') -> ['domain', 'com', ''] -> \\006domain\\0003com\\000

def octal8 (number):
	if number > 255:
		raise ValueError, 'the number does not fit in a byte (%d)', number
	return "\\%03o" % number

def octal16 (number):
	if number > 255*255:
		raise ValueError, 'the number does not fit in a 16 bits word (%d)', number

	high = number >> 8
	low = number & 0xFF
	return octal8(high) + octal8(low)

def str8 (text):
	l = len(text)
	return octal8(l)+text.replace(':','\\072')

def nstr (text):
	return "".join(str8(p) for p in dot(text).split('.'))

def naptr (domain,order,preference,flag,service,regex,replacement,ttl):
	domain = dot(domain)
	return ":%s:35:%s%s%s%s%s%s:%d\n" % (domain,octal16(order),octal16(preference),str8(flag),str8(service),str8(regex),nstr(replacement),ttl)

def udp_sip_naptr (domain,replacement,order=10,preference=100,ttl=86400,regex=''):
	return naptr (domain,order,preference,'s','SIP+D2U',regex,replacement,ttl)

def tcp_sip_naptr (domain,replacement,order=10,preference=100,ttl=86400,regex=''):
	return naptr (domain,order,preference,'s','SIP+D2T',regex,replacement,ttl)

def enum_sip_naptr (domain,sip,ttl=300,order=10,preference=100):
	return naptr (domain,order,preference,'u','E2U+sip','!^.*$!sip:'+sip+'!','',ttl)

def enum_email_naptr (domain,email,ttl=300,order=10,preference=102):
	return naptr (domain,order,preference,'u','E2U+email','!^.*$!mailto:'+email+'!','',ttl)

def srv (service,priority,weight,port,target,ttl):
	service = dot(service)
	return ":%s:33:%s%s%s%s:%d\n" % (service,octal16(priority),octal16(weight),octal16(port),nstr(target),ttl)

def sip_srv (service,target,priority=10,weight=100,ttl=86400,port=5060):
	return srv (service,priority,weight,port,target,ttl)

# Deal with +44 (0) 12345, and remove the (0)
def prefix_number (number,country='44'):
	number = number.replace(' ','')
	number = number.replace('-','')
	if number[0] == "+":	return number[1:]
	if number[:2] == "00":	return number[2:]
	if number[0] == "0":	return country+number[1:]
	return number

def enum_number (number,arpa="e164.arpa",country='44'):
	number = prefix_number(number)
	parts = [x for x in number]
	parts.reverse()
	return ".".join(parts) + "." + arpa

def to_enum (phone,email,arpa):
	line  = enum_sip_naptr(enum_number(phone,arpa),email)
	line += enum_email_naptr(enum_number(phone,arpa),email)
	return line

def make_domain (domain,ns,hostmaster,ip=''):
	domain = dot(domain)
	timestamp = time.strftime('%Y%m%d%H',time.gmtime())
	s  = "Z%s.:%s:%s:%s:3600:600:86400:3600::\n" % (domain,dot(ns[0]),hostmaster.replace('@','.'),timestamp)
	for nameserver in ns:
		s += "&%s.::%s:3600::\n" % (domain,dot(nameserver))
	if ip:
		s += "=%s:%s:3600::\n" % (domain,ip)
	#s += "@%s.::mx.exa-networks.co.uk.:20:3600::" % domain
	return s

def make_sip_domain (domain,ip,ns,hostmaster,isp='domain.exa-voice.co.uk'):
	domain = dot(domain)
	d = ''.join([domain,isp])
	u = '_sip._udp.%s' % domain
	t = '_sip._tcp.%s' % domain

	s  = make_domain (domain,ns,hostmaster)
	s += udp_sip_naptr (domain, u)
	s += sip_srv (u, d)
	s += sip_srv (t, d)
	s += "=%s:%s:3600\n" % (d,ip)
	s += make_domain ('sip.'+domain,ns,hostmaster,ip)
	s += udp_sip_naptr ('sip.'+domain, u)
	return s

if __name__ == "__main__":
	m = ":example.com.:35:\\003\\350\\000\\144\\001s\\007SIP+D2T\\033!^.*$!sip\\072info@example.com!\\004_sip\\004_tcp\\007example\\003com\\000:86400\n"
	n = naptr('example.com',1000,100,'s','SIP+D2T','!^.*$!sip:info@example.com!','_sip._tcp.example.com',86400)
	if n != m:
		print '1 - problem naptr'
		print '\tgenerated: ', n
		print '\texpecting: ', m

	m = ":example.com.:35:\\000\\012\\000\\144\\001s\\007SIP+D2T\\000\\004_sip\\004_tcp\\007example\\003com\\000:86400\n"
	n = tcp_sip_naptr ('example.com','_sip._tcp.example.com')
	if n != m:
		print '2 - problem sip_naptr'
		print '\tgenerated: ', n
		print '\texpecting: ', m

	m = ":_sip._tcp.example.com.:33:\\000\\012\\000\\144\\023\\304\\003pbx\\007example\\003com\\000:86400\n"
	n = srv ('_sip._tcp.example.com',10,100,5060,'pbx.example.com',86400)
	if n != m:
		print '3 - problem srv'
		print '\tgenerated: ', n
		print '\texpecting: ', m

	m = ":_sip._tcp.example.com.:33:\\000\\012\\000\\144\\023\\304\\003pbx\\007example\\003com\\000:86400\n"
	n = sip_srv ('_sip._tcp.example.com','pbx.example.com')
	if n != m:
		print '4 - problem sip_srv'
		print '\tgenerated: ', n
		print '\texpecting: ', m


	phone1 = "01274 61 07 93"
	phone2 = "0 0 4 4 1274 61 07 93"
	phone3 = "+ 4 4 1274 61-07-93"
	email  = "thomas.mangin@exa-networks.co.uk" 

	if not (prefix_number(phone1) == prefix_number(phone2) == prefix_number(phone3) == "441274610793"):
		print 'problem with normalising the phone number'

	if not (enum_number(phone1) == enum_number(phone2) == enum_number(phone3) == "3.9.7.0.1.6.4.7.2.1.4.4.e164.arpa"):
		print 'problem with arpaing the phone number'

	print 'if not message before, all is ok'


